# BrainGenix-NES
# AGPLv3


class Configuration():

    def __init__(self):
        
        # Create Attributes
        self.Name:str = None
        self.SourceCompartment = None
        self.ClampLocation_um:list = None
    
