# BrainGenix-NES
# AGPLv3


class Configuration():

    def __init__(self):
        
        # Create Attributes
        self.PixelResolution_nm:float = None
        self.ImageWidth_px:int = None
        self.ImageHeight_px:int = None
        self.SliceThickness_nm:float = None
        self.ScanRegionOverlap_percent:float = None 
        self.MicroscopeFOV_deg:float = None
        self.NumPixelsPerVoxel_px:int = None
    
